package sentio

import (
	"context"
	"errors"
)

var (
	ErrMarketClosed = errors.New("market is closed")
)

type Market interface {
	Connect(done chan struct{}) (chan MarketConnection, error)
	Subscribe(symbol string) error

	IsMarketOpened() bool
	Time() Clock

	Buy(ctx context.Context, symbol string, amount float64) (Order, error)
	Sell(ctx context.Context, symbol string) (Order, error)
	CancelOrder(id string) error

	Orders() ([]Order, error)
	Portfolio() (Portfolio, error)

	AvailableCash() (float64, error)
	MaxBudget() float64
}
